using System;
using DevExpress.XtraEditors;
using DevExpress.Utils;
using DevExpress.XtraEditors.Repository;
using DevExpress.XtraEditors.Registrator;
using Keyoti.RapidSpell;
using Keyoti.RapidSpell.ThirdParty.DevExpress6;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using System.Reflection;
using System.Drawing.Imaging;


/**
 * Copyright 2006 Keyoti Inc.
 * This file may be modified and redistributed at your own risk.
 * Version 1.0 - For use with Keyoti RapidSpell Desktop .NET v3.7 up.
 * Any modifications, or improvements to this are welcomed!  Please email to support@keyoti.com
 * 
 * ***************************************************************************************************************************
 * Visual Studio 2005 Users - Please find "VS2005 Uncomment" lines below and uncomment them for .NET 2 specific functionality.
 * ***************************************************************************************************************************
 * 
 * 
 * Look for "Option -" below for more options
 */

namespace Keyoti.RapidSpell.ThirdParty.DevExpress6.Glue
{
	public class GlueVars
	{
		
		public static readonly int UnderlineYOffset = 0;
		public static readonly Keyoti.RapidSpell.UnderlineStyle UnderlineStyle = Keyoti.RapidSpell.UnderlineStyle.Single;

		/* Option - wavy underlines (these may work better with bigger fonts and UnderlineYOffset=0)
		public static readonly int UnderlineYOffset = 2;
		public static readonly Keyoti.RapidSpell.UnderlineStyle UnderlineStyle = Keyoti.RapidSpell.UnderlineStyle.Wavy;
		*/
	}

    #region Grid



    /// <summary>
    /// As You Type TextEdit repository item
    /// </summary>
    [UserRepositoryItem("Register")]
    public class RepositoryItemAYTTextEdit : DevExpress.XtraEditors.Repository.RepositoryItemTextEdit
    {

        /// <summary>
        /// Shared instance of the spell checker
        /// </summary>
        public static Keyoti.RapidSpell.RapidSpellAsYouType RapidSpell;


        /// <summary>
        /// Needs to be called in Form .ctor
        /// </summary>
        public static void InitRapidSpellAsYouType()
        {
            RepositoryItemAYTTextEdit.RapidSpell = new Keyoti.RapidSpell.RapidSpellAsYouType();
            //RepositoryItemAYTTextEdit.RapidSpell.UserDictionaryFile = "user-dic.txt";
            RepositoryItemAYTTextEdit.RapidSpell.IncludeUserDictionaryInSuggestions = true;
            RepositoryItemAYTTextEdit.RapidSpell.ShowSuggestionsWhenTextIsSelected = true;
			//You may prefer to use single underlines because not much room for wavy
			RepositoryItemAYTTextEdit.RapidSpell.UnderlineStyle = GlueVars.UnderlineStyle;

            //VS2005 Uncomment
            //RepositoryItemAYTTextEdit.RapidSpell.ContextMenuStripEnabled = false;
        }

        internal const string EditorName = "AYTTextEdit";
        public static void Register()
        {

            EditorRegistrationInfo.Default.Editors.Add(
                new EditorClassInfo(EditorName, typeof(AYTTextEdit),
                typeof(RepositoryItemAYTTextEdit),
                typeof(DevExpress.XtraEditors.ViewInfo.TextEditViewInfo),
                new AYTTextEditPainter(),//uncomment for regular painter new DevExpress.XtraEditors.Drawing.TextEditPainter(), 
                true, null));

        }

        public override string EditorTypeName
        {
            get
            {
                return EditorName;
            }
        }


        static RepositoryItemAYTTextEdit()
        {
            Register();
        }

    }

    /// <summary>
    /// Paints the AYT Text Box when not in edit mode - this code draws the red "e" in the corner of the cell.
    /// </summary>
    public class AYTTextEditPainter : DevExpress.XtraEditors.Drawing.TextEditPainter
    {
        protected override void DrawString(DevExpress.XtraEditors.Drawing.ControlGraphicsInfoArgs info, Rectangle bounds, string text, DevExpress.Utils.AppearanceObject appearance)
        {
			/* Option - uncomment block for desired 'off-edit-mode' painting. */
			/* Paints an "e" in the corner -->
            if (RepositoryItemAYTTextEdit.RapidSpell != null)
            {
                //use the core spell checker from the as you type object
                Keyoti.RapidSpell.RapidSpellChecker rsc = RepositoryItemAYTTextEdit.RapidSpell.RapidSpellChecker;
                rsc.ShareDictionary = true;
                rsc.WarnDuplicates = false;

                //check the text for errors
                rsc.Check(text);

                if (rsc.NextBadWord() != null)//if errors are present then draw a red 'e' in the corner
                {
                    info.Graphics.DrawString("e", new Font("Arial", 7), System.Drawing.Brushes.Red, new PointF(info.Bounds.Right - 7, info.Bounds.Top));
                }
            }
			<--*/

			// Paints an underline -->
			if (RepositoryItemAYTTextEdit.RapidSpell != null)
			{

				Keyoti.RapidSpell.RapidSpellChecker rsc = RepositoryItemAYTTextEdit.RapidSpell.RapidSpellChecker;
				rsc.Check(text);

				Keyoti.RapidSpell.BadWord bw;
				while ((bw = rsc.NextBadWord()) != null)
				{
					int beforeWordWidth = Keyoti.RapidSpell.Grid.GridUtilities.MeasureDisplayStringWidth(info.Graphics,
						text.Substring(0, bw.StartPosition),
						appearance.Font, info.Bounds.Width, "");
					
					int wordWidth = Keyoti.RapidSpell.Grid.GridUtilities.MeasureDisplayStringWidth(info.Graphics, bw.Word, 
						info.ViewInfo.Appearance.Font,
						info.Bounds.Width,"");

					SizeF wordSize = info.Graphics.MeasureString(bw.Word, info.ViewInfo.Appearance.Font);

					Point start = new Point((int)(info.Bounds.Left + beforeWordWidth), (int)(info.Bounds.Top + wordSize.Height + 1 - GlueVars.UnderlineYOffset));
					Point end = new Point((int)(info.Bounds.Left + beforeWordWidth + wordWidth), (int)(info.Bounds.Top + wordSize.Height + 1 - GlueVars.UnderlineYOffset));

					Keyoti.RapidSpell.TextBoxUtilityAdapter.paintUnderline(info.Graphics, start, end, RepositoryItemAYTTextEdit.RapidSpell.UnderlineStyle, new Pen(RepositoryItemAYTTextEdit.RapidSpell.UnderlineColor));
				}
			}
			//<--
			base.DrawString(info, bounds, text, appearance);
        }
    }


    #endregion

    #region AYTTextEdit
    /// <summary>
	/// Summary description for AYTTextEdit.
	/// </summary>
	public class AYTTextEdit : TextEdit{

		static AYTTextEdit() 
		{            
			RepositoryItemAYTTextEdit.Register();       
		}
		public override string EditorTypeName 
		{            
			get { return RepositoryItemAYTTextEdit.EditorName; }       
		}
		public new RepositoryItemAYTTextEdit Properties 
		{      
			get { return base.Properties as RepositoryItemAYTTextEdit; }     
		}

	
		AYTTextBoxMaskBox tb;
		TextBoxConnector connector;

		public IAYTTextBox AsYouTypeTextBox
		{
			get{ return connector;}
		}
		protected override TextBoxMaskBox CreateMaskBoxInstance() 
		{
            Keyoti.RapidSpell.RapidSpellAsYouType rsayt;
            rsayt = RepositoryItemAYTTextEdit.RapidSpell;

			connector = new TextBoxConnector();
			tb = new AYTTextBoxMaskBox(this, connector);
			connector.WrappedTextBox = tb;

            rsayt.RapidSpellChecker.ShareDictionary = true;
            rsayt.TextComponent = connector;
            connector.UnderlineYOffset = GlueVars.UnderlineYOffset;


			return tb;
		}
	}
	#endregion

	#region AYTMemoEdit

	public class AYTMemoEdit: DevExpress.XtraEditors.MemoEdit
	{


		Keyoti.RapidSpell.RapidSpellAsYouType rapidSpellAsYouType1;
		protected override DevExpress.XtraEditors.TextBoxMaskBox CreateMaskBoxInstance()
		{

			rapidSpellAsYouType1 = new Keyoti.RapidSpell.RapidSpellAsYouType();
			TextBoxConnector connector = new TextBoxConnector();
			AYTTextBoxMaskBox tb= new AYTTextBoxMaskBox(this, connector);
			
			connector.WrappedTextBox = tb;
			//VS2005 Uncomment
			//rapidSpellAsYouType1.ContextMenuStripEnabled = false;
			rapidSpellAsYouType1.RapidSpellChecker.ShareDictionary = true;
			rapidSpellAsYouType1.TextComponent = connector;
			return tb;
		}

		//call when the form loads
		public void OnLoad(object s, EventArgs e)
		{
			rapidSpellAsYouType1.ForceCheckAll();
		}
	}
	#endregion

	#region Support - AYTTextBoxMaskBox
	public class AYTTextBoxMaskBox : DevExpress.XtraEditors.TextBoxMaskBox, ITextBoxMaskBox
	{
		IAYTTextBox box;

		

		///<summary>Creates a new TextBox with "as-you-type" facility.</summary>
		public AYTTextBoxMaskBox(DevExpress.XtraEditors.TextEdit ownerEdit, IAYTTextBox box) : base(ownerEdit)
		{
			this.box = box;

		}

		public override ContextMenu ContextMenu
		{
			get
			{
				return base.ContextMenu;
			}
			set
			{
				if(base.ContextMenu==null)	//required for compatibility
					base.ContextMenu = value;
			}
		}



		///<summary>Handles WndProc events and dispatches new events.</summary>
		protected override void WndProc(ref Message m)
		{
			if( (this.box as IThirdPartyTextBoxConnector).WndProc(ref m))
				base.WndProc(ref m);
		}

		public void CallWndProc(ref Message m)
		{
			base.WndProc(ref m);
		}

	
	}
	#endregion

	#region AYTMemoExEdit
	public class AYTMemoExEdit: DevExpress.XtraEditors.MemoExEdit
	{
		protected override DevExpress.XtraEditors.Popup.PopupBaseForm CreatePopupForm()
		{
			return new AYTMemoExPopupForm(this);
		}
	}

	
	public class AYTMemoExPopupForm : DevExpress.XtraEditors.Popup.MemoExPopupForm
	{
		
		AYTMemoEdit aytmemo = new AYTMemoEdit();
		public AYTMemoExPopupForm(DevExpress.XtraEditors.MemoExEdit ownerEdit) : base(ownerEdit)
		{	

			aytmemo.ContextMenu = new ContextMenu();
			if (this.OwnerEdit.LookAndFeel.ActiveStyle != DevExpress.LookAndFeel.ActiveLookAndFeelStyle.Skin)
			{
				this.aytmemo.Properties.BorderStyle = DevExpress.XtraEditors.Controls.BorderStyles.Simple;
			}
			this.aytmemo.Properties.Appearance.Assign(ownerEdit.Properties.AppearanceDropDown);
			this.aytmemo.Visible = false;
			this.aytmemo.Modified += new EventHandler(this.OnMemo_Modified);
			base.Controls.Add(this.aytmemo);
			this.UpdateMemo();

		}


		protected override DevExpress.XtraEditors.MemoEdit Memo
		{
			get
			{
				return this.aytmemo;
			}
		}
 
	}
	#endregion

}
